// Playground - noun: a place where people can play

import UIKit

/////////////////////////////////////////
// code 3.80,3.81
/////////////////////////////////////////
protocol AlphaProtocol {
    func print()
}
protocol BetaProtocol {}
class SomeClass {}

class C:SomeClass, AlphaProtocol {
    func print() {
        println("I'm class")
    }
}
struct S:AlphaProtocol, BetaProtocol {
    func print() {
        println("I'm struct")
    }
}
enum E:AlphaProtocol {
    func print() {
        println("I'm enum")
    }
}

/////////////////////////////////////////
// code 3.82
/////////////////////////////////////////
protocol InitializeProtocol {
    init(str:String)
}
class C2:InitializeProtocol {
    required init(str: String) {
        // implementation
    }
}

/////////////////////////////////////////
// code 3.83
/////////////////////////////////////////
protocol AlphaProtocol2 {
    var printString:String {get set}
    func print()
}

/////////////////////////////////////////
// code 3.84,3.85
/////////////////////////////////////////
protocol ClassFuncProtocol {
    class func myType() -> String
}
class C3:ClassFuncProtocol {
    class func myType() -> String {
        return "Class"
    }
}
struct S3:ClassFuncProtocol {
    static func myType() -> String {
        return "Struct"
    }
}
enum E3:ClassFuncProtocol {
    static func myType() -> String {
        return "Enum"
    }
}

/////////////////////////////////////////
// code 3.86
/////////////////////////////////////////
protocol InclementProtocol {
    var i:Int {get}
    mutating func inclement()
}
class C4: InclementProtocol {
    var i = 0
    func inclement() {
        i++
    }
}
struct S4: InclementProtocol {
    var i = 0
    mutating func inclement() {
        i++
    }
}

/////////////////////////////////////////
// code 3.90
/////////////////////////////////////////
protocol SomeProtocol {}
protocol Some2Protocol {}
class SomeProtocolImpl: SomeProtocol, Some2Protocol {}
var a:SomeProtocol = SomeProtocolImpl()
var collection:[SomeProtocol] = []
func someMethod(val:SomeProtocol) {}
var c5:protocol<SomeProtocol, Some2Protocol> = SomeProtocolImpl()

/////////////////////////////////////////
// code 3.92
/////////////////////////////////////////
class SomeType {}
extension SomeType {
    func appendMethod() {
        // implementation
    }
}
/////////////////////////////////////////
// code 3.94
/////////////////////////////////////////
struct Rect {
    var x:Double, y:Double, width:Double, height:Double
}
extension Rect {
    var top:Double {
        get {
            return y
        }
        set {
            y = newValue
        }
    }
    var bottom:Double {
        get {
            return y + height
        }
        set {
            y = newValue - height
        }
    }
}
extension Rect {
    static var zeroRect:Rect {
        get {
            return Rect(x: 0, y: 0, width: 0, height: 0)
        }
    }
}
var rect = Rect.zeroRect
rect.height = 100
println(rect.bottom)
rect.bottom = 200
println(rect.top)

/////////////////////////////////////////
// code 3.95
/////////////////////////////////////////
extension Rect {
    init(x:Double, y:Double, squareSideLength sideLength:Double) {
        self.init(x:x, y:y, width:sideLength, height:sideLength)
    }
}
var square = Rect(x: 0, y: 0, squareSideLength: 200)

/////////////////////////////////////////
// code 3.96
/////////////////////////////////////////
extension Rect {
    func area() -> Double {
        return width * height
    }
    mutating func move(x:Double, y:Double) {
        self.x += x
        self.y += y
    }
}

/////////////////////////////////////////
// code 3.97
/////////////////////////////////////////
let str = "Hello Swift"
extension String {
    subscript(index:Int) -> String? {
        if index >= countElements(self) {
            return nil
        }
        let strStartIndex = advance(self.startIndex, index)
        let strEndIndex = advance(strStartIndex, 1)
        return self.substringWithRange(strStartIndex..<strEndIndex)
    }
}
println(str[6]) // "S"

/////////////////////////////////////////
// code 3.98
/////////////////////////////////////////
extension Rect {
    enum Kind {
        case NoArea, Squere, Portrait, Landscape
    }
    var kind:Kind {
        get {
            if self.area() == 0 {
                return .NoArea
            }
            if self.width == self.height {
                return .Squere
            }
            if self.width < self.height {
                return .Portrait
            }
            return .Landscape
        }
    }
}
var rect2 = Rect(x: 0, y: 0, width: 100, height: 200)
rect2.kind // Rect.kind.Potrait


